@@ -4,16 +4,16 @@
 # This work is licensed under the GNU GPLv2 or later.
 # See the COPYING file in the top-level directory.
 
-import logging
-
 from gi.repository import Gtk
 from gi.repository import Gdk
 
 from virtinst import DomainCpu
+from virtinst import log
 
-from . import uiutil
+from .lib import uiutil
 from .baseclass import vmmGObjectUI
-from .inspection import vmmInspection
+from .lib.inspection import vmmInspection
+from .systray import vmmSystray
 
 
 class vmmPreferences(vmmGObjectUI):
@@ -23,7 +23,7 @@ class vmmPreferences(vmmGObjectUI):
             if not cls._instance:
                 cls._instance = vmmPreferences()
             cls._instance.show(parentobj.topwin)
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             parentobj.err.show_err(
                     _("Error launching preferences: %s") % str(e))
 
@@ -36,17 +36,17 @@ class vmmPreferences(vmmGObjectUI):
         self._orig_libguestfs_val = None
 
         self.refresh_view_system_tray()
+        self.refresh_xmleditor()
         self.refresh_libguestfs()
         self.refresh_update_interval()
-        self.refresh_console_accels()
         self.refresh_console_scaling()
         self.refresh_console_resizeguest()
         self.refresh_console_autoredir()
-        self.refresh_new_vm_sound()
+        self.refresh_console_autoconnect()
         self.refresh_graphics_type()
-        self.refresh_add_spice_usbredir()
         self.refresh_storage_format()
         self.refresh_cpu_default()
+        self.refresh_firmware_default()
         self.refresh_cpu_poll()
         self.refresh_disk_poll()
         self.refresh_net_poll()
@@ -56,7 +56,6 @@ class vmmPreferences(vmmGObjectUI):
         self.refresh_confirm_poweroff()
         self.refresh_confirm_pause()
         self.refresh_confirm_removedev()
-        self.refresh_confirm_interface()
         self.refresh_confirm_unapplied()
         self.refresh_confirm_delstorage()
 
@@ -65,17 +64,17 @@ class vmmPreferences(vmmGObjectUI):
             "on_prefs_close_clicked": self.close,
 
             "on_prefs_system_tray_toggled": self.change_view_system_tray,
+            "on_prefs_xmleditor_toggled": self.change_xmleditor,
             "on_prefs_libguestfs_toggled": self.change_libguestfs,
             "on_prefs_stats_update_interval_changed": self.change_update_interval,
-            "on_prefs_console_accels_toggled": self.change_console_accels,
             "on_prefs_console_scaling_changed": self.change_console_scaling,
             "on_prefs_console_resizeguest_changed": self.change_console_resizeguest,
             "on_prefs_console_autoredir_changed": self.change_console_autoredir,
-            "on_prefs_new_vm_sound_toggled": self.change_new_vm_sound,
+            "on_prefs_console_autoconnect_toggled": self.change_console_autoconnect,
             "on_prefs_graphics_type_changed": self.change_graphics_type,
-            "on_prefs_add_spice_usbredir_changed": self.change_add_spice_usbredir,
             "on_prefs_storage_format_changed": self.change_storage_format,
             "on_prefs_cpu_default_changed": self.change_cpu_default,
+            "on_prefs_firmware_default_changed": self.change_firmware_default,
             "on_prefs_stats_enable_cpu_toggled": self.change_cpu_poll,
             "on_prefs_stats_enable_disk_toggled": self.change_disk_poll,
             "on_prefs_stats_enable_net_toggled": self.change_net_poll,
@@ -84,7 +83,6 @@ class vmmPreferences(vmmGObjectUI):
             "on_prefs_confirm_poweroff_toggled": self.change_confirm_poweroff,
             "on_prefs_confirm_pause_toggled": self.change_confirm_pause,
             "on_prefs_confirm_removedev_toggled": self.change_confirm_removedev,
-            "on_prefs_confirm_interface_toggled": self.change_confirm_interface,
             "on_prefs_confirm_unapplied_toggled": self.change_confirm_unapplied,
             "on_prefs_confirm_delstorage_toggled": self.change_confirm_delstorage,
             "on_prefs_btn_keys_define_clicked": self.change_grab_keys,
@@ -95,12 +93,12 @@ class vmmPreferences(vmmGObjectUI):
         self.bind_escape_key_close()
 
     def close(self, ignore1=None, ignore2=None):
-        logging.debug("Closing preferences")
+        log.debug("Closing preferences")
         self.topwin.hide()
         return 1
 
     def show(self, parent):
-        logging.debug("Showing preferences")
+        log.debug("Showing preferences")
         self.topwin.set_transient_for(parent)
         self.topwin.present()
 
@@ -154,16 +152,6 @@ class vmmPreferences(vmmGObjectUI):
         combo.set_model(model)
         uiutil.init_combo_text_column(combo, 1)
 
-        combo = self.widget("prefs-add-spice-usbredir")
-        # [gsettings value, string]
-        model = Gtk.ListStore(str, str)
-        for row in [["system", _("System default (%s)") %
-                     self.config.default_add_spice_usbredir],
-                    ["yes", _("Yes")], ["no", _("No")]]:
-            model.append(row)
-        combo.set_model(model)
-        uiutil.init_combo_text_column(combo, 1)
-
         combo = self.widget("prefs-storage-format")
         # [gsettings value, string]
         model = Gtk.ListStore(str, str)
@@ -183,13 +171,22 @@ class vmmPreferences(vmmGObjectUI):
             [DomainCpu.SPECIAL_MODE_HV_DEFAULT, _("Hypervisor default")],
             [DomainCpu.SPECIAL_MODE_HOST_MODEL_ONLY,
                 _("Nearest host CPU model")],
-            [DomainCpu.SPECIAL_MODE_HOST_MODEL,
-                _("Copy host CPU definition")]]:
+            [DomainCpu.SPECIAL_MODE_HOST_MODEL, "host-model"],
+            [DomainCpu.SPECIAL_MODE_HOST_PASSTHROUGH, "host-passthrough"]]:
             model.append(row)
         combo.set_model(model)
         uiutil.init_combo_text_column(combo, 1)
 
-        if not vmmInspection.libguestfs_installed():
+        combo = self.widget("prefs-firmware-default")
+        # [gsettings value, string]
+        model = Gtk.ListStore(str, str)
+        for row in [["default", _("System default")],
+                    ["uefi", "UEFI"]]:
+            model.append(row)
+        combo.set_model(model)
+        uiutil.init_combo_text_column(combo, 1)
+
+        if not vmmInspection.libguestfs_installed():  # pragma: no cover
             self.widget("prefs-libguestfs").set_sensitive(False)
             self.widget("prefs-libguestfs").set_tooltip_text(
                     _("python libguestfs support is not installed"))
@@ -200,8 +197,15 @@ class vmmPreferences(vmmGObjectUI):
     #########################
 
     def refresh_view_system_tray(self):
-        val = self.config.get_view_system_tray()
-        self.widget("prefs-system-tray").set_active(bool(val))
+        errmsg = vmmSystray.systray_disabled_message()
+        val = bool(self.config.get_view_system_tray() and not errmsg)
+        self.widget("prefs-system-tray").set_sensitive(not bool(errmsg))
+        self.widget("prefs-system-tray").set_tooltip_text(errmsg)
+        self.widget("prefs-system-tray").set_active(val)
+
+    def refresh_xmleditor(self):
+        val = self.config.get_xmleditor_enabled()
+        self.widget("prefs-xmleditor").set_active(bool(val))
 
     def refresh_libguestfs(self):
         val = self.config.get_libguestfs_inspect_vms()
@@ -213,9 +217,6 @@ class vmmPreferences(vmmGObjectUI):
         self.widget("prefs-stats-update-interval").set_value(
             self.config.get_stats_update_interval())
 
-    def refresh_console_accels(self):
-        self.widget("prefs-console-accels").set_active(
-            self.config.get_console_accels())
     def refresh_console_scaling(self):
         combo = self.widget("prefs-console-scaling")
         val = self.config.get_console_scaling()
@@ -228,18 +229,14 @@ class vmmPreferences(vmmGObjectUI):
         combo = self.widget("prefs-console-autoredir")
         val = self.config.get_auto_usbredir()
         uiutil.set_list_selection(combo, val)
+    def refresh_console_autoconnect(self):
+        val = self.config.get_console_autoconnect()
+        self.widget("prefs-console-autoconnect").set_active(val)
 
-    def refresh_new_vm_sound(self):
-        self.widget("prefs-new-vm-sound").set_active(
-            self.config.get_new_vm_sound())
     def refresh_graphics_type(self):
         combo = self.widget("prefs-graphics-type")
         gtype = self.config.get_graphics_type(raw=True)
         uiutil.set_list_selection(combo, gtype)
-    def refresh_add_spice_usbredir(self):
-        combo = self.widget("prefs-add-spice-usbredir")
-        val = self.config.get_add_spice_usbredir(raw=True)
-        uiutil.set_list_selection(combo, val)
     def refresh_storage_format(self):
         combo = self.widget("prefs-storage-format")
         val = self.config.get_default_storage_format(raw=True)
@@ -248,6 +245,10 @@ class vmmPreferences(vmmGObjectUI):
         combo = self.widget("prefs-cpu-default")
         val = self.config.get_default_cpu_setting()
         uiutil.set_list_selection(combo, val)
+    def refresh_firmware_default(self):
+        combo = self.widget("prefs-firmware-default")
+        val = self.config.get_default_firmware_setting()
+        uiutil.set_list_selection(combo, val)
 
     def refresh_cpu_poll(self):
         self.widget("prefs-stats-enable-cpu").set_active(
@@ -262,28 +263,21 @@ class vmmPreferences(vmmGObjectUI):
         self.widget("prefs-stats-enable-memory").set_active(
             self.config.get_stats_enable_memory_poll())
 
+    def _process_grabkeys(self, val):
+        # We convert keysyms to names
+        keys = []
+        for k in val.split(','):
+            try:
+                key = int(k)
+                keys.append(Gdk.keyval_name(key))
+            except Exception:  # pragma: no cover
+                continue
+        return "+".join(keys)
+
     def refresh_grabkeys_combination(self):
         val = self.config.get_keys_combination()
-
-        # We convert keysyms to names
-        if not val:
-            keystr = "Control_L+Alt_L"
-        else:
-            keystr = None
-            for k in val.split(','):
-                try:
-                    key = int(k)
-                except Exception:
-                    key = None
-
-                if key is not None:
-                    if keystr is None:
-                        keystr = Gdk.keyval_name(key)
-                    else:
-                        keystr = keystr + "+" + Gdk.keyval_name(key)
-
-
-        self.widget("prefs-keys-grab-sequence").set_text(keystr)
+        keystr = self._process_grabkeys(val)
+        self.widget("prefs-keys-grab-sequence").set_text(keystr or "")
 
     def refresh_confirm_forcepoweroff(self):
         self.widget("prefs-confirm-forcepoweroff").set_active(
@@ -297,9 +291,6 @@ class vmmPreferences(vmmGObjectUI):
     def refresh_confirm_removedev(self):
         self.widget("prefs-confirm-removedev").set_active(
                                 self.config.get_confirm_removedev())
-    def refresh_confirm_interface(self):
-        self.widget("prefs-confirm-interface").set_active(
-                                self.config.get_confirm_interface())
     def refresh_confirm_unapplied(self):
         self.widget("prefs-confirm-unapplied").set_active(
                                 self.config.get_confirm_unapplied())
@@ -364,6 +355,8 @@ class vmmPreferences(vmmGObjectUI):
 
     def change_view_system_tray(self, src):
         self.config.set_view_system_tray(src.get_active())
+    def change_xmleditor(self, src):
+        self.config.set_xmleditor_enabled(src.get_active())
     def change_libguestfs(self, src):
         val = src.get_active()
         self.config.set_libguestfs_inspect_vms(val)
@@ -376,8 +369,6 @@ class vmmPreferences(vmmGObjectUI):
     def change_update_interval(self, src):
         self.config.set_stats_update_interval(src.get_value_as_int())
 
-    def change_console_accels(self, src):
-        self.config.set_console_accels(src.get_active())
     def change_console_scaling(self, box):
         self.config.set_console_scaling(box.get_active())
     def change_console_resizeguest(self, box):
@@ -386,23 +377,21 @@ class vmmPreferences(vmmGObjectUI):
     def change_console_autoredir(self, box):
         val = uiutil.get_list_selection(box)
         self.config.set_auto_usbredir(val)
+    def change_console_autoconnect(self, src):
+        self.config.set_console_autoconnect(bool(src.get_active()))
 
-    def change_new_vm_sound(self, src):
-        self.config.set_new_vm_sound(src.get_active())
     def change_graphics_type(self, src):
         val = uiutil.get_list_selection(src)
         self.config.set_graphics_type(val)
-        uiutil.set_grid_row_visible(
-            self.widget("prefs-add-spice-usbredir"),
-            self.config.get_graphics_type() == "spice")
-    def change_add_spice_usbredir(self, src):
-        self.config.set_add_spice_usbredir(uiutil.get_list_selection(src))
     def change_storage_format(self, src):
         typ = uiutil.get_list_selection(src) or "default"
         self.config.set_storage_format(typ.lower())
     def change_cpu_default(self, src):
         typ = uiutil.get_list_selection(src) or "default"
         self.config.set_default_cpu_setting(typ.lower())
+    def change_firmware_default(self, src):
+        typ = uiutil.get_list_selection(src) or "default"
+        self.config.set_firmware_setting(typ.lower())
 
     def change_cpu_poll(self, src):
         self.config.set_stats_enable_cpu_poll(src.get_active())
@@ -421,8 +410,6 @@ class vmmPreferences(vmmGObjectUI):
         self.config.set_confirm_pause(src.get_active())
     def change_confirm_removedev(self, src):
         self.config.set_confirm_removedev(src.get_active())
-    def change_confirm_interface(self, src):
-        self.config.set_confirm_interface(src.get_active())
     def change_confirm_unapplied(self, src):
         self.config.set_confirm_unapplied(src.get_active())
     def change_confirm_delstorage(self, src):
