@@ -49,7 +49,7 @@ class _StorageObject(XMLBuilder):
 
 def _preferred_default_pool_path(conn):
     path = "/var/lib/libvirt/images"
-    if conn.is_session_uri():
+    if conn.is_unprivileged():
         path = os.path.expanduser("~/.local/share/libvirt/images")
     return path
 
@@ -162,9 +162,10 @@ class StoragePool(_StorageObject):
             defpool.install(build=True, create=True, autostart=True)
             return defpool
         except Exception as e:  # pragma: no cover
-            raise RuntimeError(
-                _("Couldn't create default storage pool '%s': %s") %
-                (path, str(e)))
+            log.debug("Error building default pool", exc_info=True)
+            msg = (_("Couldn't create default storage pool '%(path)s': %(error)s") %
+                    {"path": path, "error": str(e)})
+            raise RuntimeError(msg) from None
 
     @staticmethod
     def lookup_pool_by_path(conn, path):
@@ -384,7 +385,8 @@ class StoragePool(_StorageObject):
         try:
             pool = self.conn.storagePoolDefineXML(xml, 0)
         except Exception as e:  # pragma: no cover
-            raise RuntimeError(_("Could not define storage pool: %s") % str(e))
+            msg = _("Could not define storage pool: %s") % str(e)
+            raise RuntimeError(msg) from None
 
         errmsg = None
         if build:
@@ -419,6 +421,31 @@ class StoragePool(_StorageObject):
         return pool
 
 
+def _progress_thread(volname, pool, meter, event):
+    vol = None
+    if not meter:
+        return  # pragma: no cover
+
+    while True:
+        try:
+            if not vol:
+                vol = pool.storageVolLookupByName(volname)
+            vol.info()  # pragma: no cover
+            break  # pragma: no cover
+        except Exception:
+            if event.wait(.2):
+                break
+
+    if vol is None:
+        log.debug("Couldn't lookup storage volume in prog thread.")
+        return
+
+    while True:  # pragma: no cover
+        dummy1, dummy2, alloc = vol.info()
+        meter.update(alloc)
+        if event.wait(1):
+            break
+
 
 class StorageVolume(_StorageObject):
     """
@@ -451,9 +478,10 @@ class StorageVolume(_StorageObject):
                     break
 
             for disk in collideguest.devices.disk:
-                if (pooltarget and disk.path and
-                    os.path.dirname(disk.path) == pooltarget):
-                    collidelist.append(os.path.basename(disk.path))
+                checkpath = disk.get_source_path()
+                if (pooltarget and checkpath and
+                    os.path.dirname(checkpath) == pooltarget):
+                    collidelist.append(os.path.basename(checkpath))
 
         def cb(tryname):
             if tryname in collidelist:
@@ -479,8 +507,6 @@ class StorageVolume(_StorageObject):
         self._pool_xml = None
         self._reflink = False
 
-        self._install_finished = threading.Event()
-
 
     ######################
     # Non XML properties #
@@ -495,29 +521,25 @@ class StorageVolume(_StorageObject):
             parsexml=self._pool.XMLDesc(0))
     pool = property(_get_pool, _set_pool)
 
-    def _get_input_vol(self):
+    @property
+    def input_vol(self):
         return self._input_vol
-    def _set_input_vol(self, vol):
+    def set_input_vol(self, vol):
         self._input_vol = vol
-    input_vol = property(_get_input_vol, _set_input_vol)
-
-    def _get_reflink(self):
-        return self._reflink
-    def _set_reflink(self, reflink):
-        self._reflink = reflink
-    reflink = property(_get_reflink, _set_reflink)
-
-    def sync_input_vol(self, only_format=False):
-        # Pull parameters from input vol into this class
         parsevol = StorageVolume(self.conn,
                                  parsexml=self._input_vol.XMLDesc(0))
 
         self.format = parsevol.format
         self.capacity = parsevol.capacity
         self.allocation = parsevol.allocation
-        if only_format:
-            return
-        self.pool = self._input_vol.storagePoolLookupByVolume()
+        if not self._pool:
+            self.pool = self._input_vol.storagePoolLookupByVolume()
+
+    def _get_reflink(self):
+        return self._reflink
+    def _set_reflink(self, reflink):
+        self._reflink = reflink
+    reflink = property(_get_reflink, _set_reflink)
 
 
     ##########################
@@ -573,7 +595,8 @@ class StorageVolume(_StorageObject):
         log.debug("Attempting to detect format for backing_store=%s",
                 self.backing_store)
         from . import diskbackend
-        vol, pool = diskbackend.manage_path(self.conn, self.backing_store)
+        path, vol, pool = diskbackend.manage_path(self.conn, self.backing_store)
+        dummy = path
 
         if not vol:  # pragma: no cover
             log.debug("Didn't find any volume for backing_store")
@@ -600,11 +623,7 @@ class StorageVolume(_StorageObject):
     ######################
 
     def supports_format(self):
-        if self.file_type == self.TYPE_FILE:
-            return True
-        if self._pool_xml.type == StoragePool.TYPE_GLUSTER:
-            return True
-        return False
+        return self.file_type == self.TYPE_FILE
 
 
     ##################
@@ -642,13 +661,6 @@ class StorageVolume(_StorageObject):
         log.debug("Creating storage volume '%s' with xml:\n%s",
                       self.name, xml)
 
-        t = threading.Thread(target=self._progress_thread,
-                             name="Checking storage allocation",
-                             args=(meter,))
-        t.setDaemon(True)
-
-        meter = progress.ensure_meter(meter)
-
         cloneflags = 0
         createflags = 0
         if (self.format == "qcow2" and
@@ -664,11 +676,17 @@ class StorageVolume(_StorageObject):
             cloneflags |= getattr(libvirt,
                 "VIR_STORAGE_VOL_CREATE_REFLINK", 1)
 
+        event = threading.Event()
+        meter = progress.ensure_meter(meter)
+        t = threading.Thread(target=_progress_thread,
+                             name="Checking storage allocation",
+                             args=(self.name, self.pool, meter, event))
+        t.daemon = True
+
         try:
-            self._install_finished.clear()
             t.start()
-            meter.start(size=self.capacity,
-                        text=_("Allocating '%s'") % self.name)
+            msg = _("Allocating '%(filename)s'") % {"filename": self.name}
+            meter.start(msg, self.capacity)
 
             if self.conn.is_really_test():
                 # Test suite doesn't support any flags, so reset them
@@ -681,41 +699,17 @@ class StorageVolume(_StorageObject):
                 log.debug("Using vol create flags=%s", createflags)
                 vol = self.pool.createXML(xml, createflags)
 
-            self._install_finished.set()
-            t.join()
-            meter.end(self.capacity)
+            meter.end()
             log.debug("Storage volume '%s' install complete.", self.name)
             return vol
         except Exception as e:
             log.debug("Error creating storage volume", exc_info=True)
-            raise RuntimeError("Couldn't create storage volume "
-                               "'%s': '%s'" % (self.name, str(e)))
-
-    def _progress_thread(self, meter):
-        vol = None
-        if not meter:
-            return
-
-        while True:
-            try:
-                if not vol:
-                    vol = self.pool.storageVolLookupByName(self.name)
-                vol.info()  # pragma: no cover
-                break  # pragma: no cover
-            except Exception:
-                if self._install_finished.wait(.2):
-                    break
-
-        if vol is None:
-            log.debug("Couldn't lookup storage volume in prog thread.")
-            return
-
-        while True:  # pragma: no cover
-            ignore, ignore, alloc = vol.info()
-            meter.update(alloc)
-            if self._install_finished.wait(1):
-                break
-
+            msg = ("Couldn't create storage volume '%s': '%s'" % (
+                self.name, str(e)))
+            raise RuntimeError(msg) from None
+        finally:
+            event.set()
+            t.join()
 
     def is_size_conflict(self):
         """
@@ -731,16 +725,18 @@ class StorageVolume(_StorageObject):
         # pool info is [pool state, capacity, allocation, available]
         avail = self.pool.info()[3]
         if self.allocation > avail:
-            return (True, _("There is not enough free space on the storage "
-                            "pool to create the volume. "
-                            "(%d M requested allocation > %d M available)") %
-                            ((self.allocation // (1024 * 1024)),
-                             (avail // (1024 * 1024))))
+            msg = (_("There is not enough free space on the storage "
+                    "pool to create the volume. (%(mem1)s M requested "
+                    "allocation > %(mem2)s M available)") %
+                    {"mem1": (self.allocation // (1024 * 1024)),
+                     "mem2": (avail // (1024 * 1024))})
+            return (True, msg)
         elif self.capacity > avail:
-            return (False, _("The requested volume capacity will exceed the "
-                             "available pool space when the volume is fully "
-                             "allocated. "
-                             "(%d M requested capacity > %d M available)") %
-                             ((self.capacity // (1024 * 1024)),
-                              (avail // (1024 * 1024))))
+            msg = (_("The requested volume capacity will exceed the "
+                     "available pool space when the volume is fully "
+                     "allocated. (%(mem1)s M requested "
+                     "capacity > %(mem2)s M available)") %
+                     {"mem1": (self.capacity // (1024 * 1024)),
+                      "mem2": (avail // (1024 * 1024))})
+            return (False, msg)
         return (False, "")
